@@ -21,10 +21,13 @@ type NomadInstaller struct {
 }
 
 type nomadConfig struct {
-	authSoftFail       bool              `hcl:"auth_soft_fail,optional"`
-	serverImage        string            `hcl:"server_image,optional"`
-	namespace          string            `hcl:"namespace,optional"`
-	serviceAnnotations map[string]string `hcl:"service_annotations,optional"`
+	authSoftFail             bool              `hcl:"auth_soft_fail,optional"`
+	serverImage              string            `hcl:"server_image,optional"`
+	namespace                string            `hcl:"namespace,optional"`
+	serviceAnnotations       map[string]string `hcl:"service_annotations,optional"`
+	consulService            bool              `hcl:"consul_service,optional"`
+	consulServiceUITags      []string          `hcl:"consul_service_ui_tags:optional"`
+	consulServiceBackendTags []string          `hcl:"consul_service_backend_tags:optional"`
 
 	region         string   `hcl:"namespace,optional"`
 	datacenters    []string `hcl:"datacenters,optional"`
@@ -622,6 +625,23 @@ func waypointNomadJob(c nomadConfig) *api.Job {
 	grpcPort, _ := strconv.Atoi(defaultGrpcPort)
 	httpPort, _ := strconv.Atoi(defaultHttpPort)
 
+	// Include services to be registered in Consul, if specified in the server install
+	// One service added for Waypoint UI, and one for Waypoint backend port
+	if c.consulService {
+		tg.Services = []*api.Service{
+			{
+				Name:      "waypoint-ui",
+				PortLabel: "ui",
+				Tags:      c.consulServiceUITags,
+			},
+			{
+				Name:      "waypoint-server",
+				PortLabel: "server",
+				Tags:      c.consulServiceBackendTags,
+			},
+		}
+	}
+
 	tg.Networks = []*api.NetworkResource{
 		{
 			Mode: "host",
@@ -849,6 +869,25 @@ func (i *NomadInstaller) InstallFlags(set *flag.Set) {
 		Usage:   "Docker image for the Waypoint server.",
 		Default: defaultServerImage,
 	})
+
+	set.BoolVar(&flag.BoolVar{
+		Name:    "nomad-consul-service",
+		Target:  &i.config.consulService,
+		Usage:   "Create service for Waypoint UI in Consul.",
+		Default: false,
+	})
+
+	set.StringSliceVar(&flag.StringSliceVar{
+		Name:   "nomad-consul-service-ui-tags",
+		Target: &i.config.consulServiceUITags,
+		Usage:  "Tags for the Waypoint UI service generated in Consul.",
+	})
+
+	set.StringSliceVar(&flag.StringSliceVar{
+		Name:   "nomad-consul-service-backend-tags",
+		Target: &i.config.consulServiceBackendTags,
+		Usage:  "Tags for the Waypoint backend service generated in Consul.",
+	})
 }
 
 func (i *NomadInstaller) UpgradeFlags(set *flag.Set) {
