@@ -36,6 +36,7 @@ export default function (this: Server): void {
   this.post('/GetBuild', build.get);
   this.post('/ListDeployments', deployment.list);
   this.post('/GetDeployment', deployment.get);
+  this.post('/UpsertProject', project.update);
   this.post('/ListProjects', project.list);
   this.post('/GetProject', project.get);
   this.post('/ConvertInviteToken', token.create);
@@ -49,6 +50,7 @@ export default function (this: Server): void {
   this.post('/GetJobStream', job.stream);
   this.post('/GetLogStream', log.stream);
   this.post('/ListPushedArtifacts', pushedArtifact.list);
+  this.post('/ExpediteStatusReport', statusReport.expediteStatusReport);
 
   if (!Ember.testing) {
     // Pass through all other requests
